/**
 * Class to recursively draw broccoli
 * 
 * Williams College CSC 134
 * Updated by Jim Teresco for CSC 252, The College of Saint Rose, Fall 2013
 * 
 * $Id: BroccoliBranch.java 2244 2013-11-06 04:54:22Z terescoj $

 */
import objectdraw.*;
import java.awt.*;

public class BroccoliBranch implements BroccoliPart {
    // dark green color for broccoli
    private static final Color BROCCOLI_COLOR = new Color(0,135,0);

    // Below this size draw flower
    private static final double MINSIZE = 25.0;

    // How much broccoli shrinks each call
    private final static double TOP_FRACT = 0.8;

    // stem of broccoli
    private AngLine stem;

    // branches of broccoli
    private BroccoliPart left, center, right;

    /**
     * Draw broccoli by recursively drawing branches (and flower)
     */
    public BroccoliBranch(Location startCoords, double size,
    double direction, DrawingCanvas canvas)
    {
        // Draw stem and color green
        stem = new AngLine(startCoords, size, direction, canvas);
        stem.setColor(BROCCOLI_COLOR);

        Location destCoords = stem.getEnd();    // end of stem

        if (size > MINSIZE) {
            left = new BroccoliBranch(destCoords, size * TOP_FRACT,
                direction + Math.PI/9.0, canvas);
            center = new BroccoliBranch(destCoords, size * TOP_FRACT,
                direction, canvas);
            right = new BroccoliBranch(destCoords, size * TOP_FRACT,
                direction - Math.PI/9.0, canvas);
        } 
        else {
            left = new Flower(destCoords, size * TOP_FRACT,
                direction + Math.PI/9.0, canvas);
            center = new Flower(destCoords, size * TOP_FRACT,
                direction, canvas);
            right = new Flower(destCoords, size * TOP_FRACT,
                direction - Math.PI/9.0, canvas);
        }

    }

    /**
     * @param x,y amount to move broccoli
     */
    public void move(double x, double y) {

        stem.move(x,y);                 // move stem

        left.move(x,y);                 // move other parts
        center.move(x,y);
        right.move(x,y);
    }

    /**
     * @param point location to be checked for containment in broccoli
     */
    public boolean contains(Location point) {

        return (stem.contains(point) || left.contains(point) ||
            center.contains(point) || right.contains(point));
    }

}